{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% load url from future %}

{% block form_id %}create_policy_form{% endblock %}
{% block form_action %}{% url 'horizon:admin:policies:create' %}{% endblock %}

{% block modal_id %}create_policy_modal{% endblock %}
{% block modal-header %}{% trans "Create Policy" %}{% endblock %}

{% block modal-body %}
<div class="left">
  <fieldset>
    {% include "horizon/common/_form_fields.html" %}
  </fieldset>
</div>
{% endblock %}

{% block modal-footer %}
  <input class="btn btn-primary pull-right" type="submit" value="{% trans "Create Policy" %}" />
  <a href="{% url 'horizon:admin:policies:index' %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
