PANEL = 'policies'
PANEL_DASHBOARD = 'admin'
PANEL_GROUP = 'policy'
ADD_PANEL = 'congress_dashboard.policies.panel.Policies'
ADD_INSTALLED_APPS = [
    'congress_dashboard',
]
AUTO_DISCOVER_STATIC_FILES = True
ADD_SCSS_FILES = ['congress_dashboard/static/admin/css/policies.css']
